"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const types_1 = require("./types");
const helpers_1 = require("../helpers");
const client_1 = require("@apollo/client");
const types_2 = require("./types");
/**
 * Executes invalidation policies for types when they are modified, evicted or read from the cache.
 */
class InvalidationPolicyManager {
    constructor(config) {
        this.config = config;
        const { cacheOperations: { readField, evict, modify }, } = this.config;
        // Watch broadcasts by evict and modify operations called by policy actions
        // are suppressed until after all policy actions have run.
        this.mutedCacheOperations = {
            readField,
            evict: (options) => evict(Object.assign(Object.assign({}, options), { broadcast: false })),
            modify: (options) => modify(Object.assign(Object.assign({}, options), { broadcast: false })),
        };
        this.policyActivation = this.activatePolicies();
    }
    activatePolicies() {
        const { policies } = this.config;
        const { types: policyTypes = {}, timeToLive: defaultTimeToLive } = policies;
        return Object.keys(policyTypes).reduce((acc, type) => {
            const policy = policyTypes[type];
            acc[types_1.InvalidationPolicyEvent.Read] =
                acc[types_1.InvalidationPolicyEvent.Read] || !!policy.timeToLive;
            acc[types_1.InvalidationPolicyEvent.Write] =
                acc[types_1.InvalidationPolicyEvent.Write] ||
                    !!policy[types_1.InvalidationPolicyLifecycleEvent.Write];
            acc[types_1.InvalidationPolicyEvent.Evict] =
                acc[types_1.InvalidationPolicyEvent.Evict] ||
                    !!policy[types_1.InvalidationPolicyLifecycleEvent.Evict];
            return acc;
        }, {
            [types_1.InvalidationPolicyEvent.Read]: !!defaultTimeToLive,
            [types_1.InvalidationPolicyEvent.Write]: false,
            [types_1.InvalidationPolicyEvent.Evict]: false,
        });
    }
    getPolicy(typeName) {
        var _a, _b;
        return ((_b = (_a = this.config.policies) === null || _a === void 0 ? void 0 : _a.types) === null || _b === void 0 ? void 0 : _b[typeName]) || null;
    }
    getTypePolicyForEvent(typeName, policyEvent) {
        const policyForType = this.getPolicy(typeName);
        if (!policyForType) {
            return null;
        }
        return policyForType[types_1.InvalidationPolicyLifecycleEvent[policyEvent]];
    }
    runPolicyEvent(typeName, policyEvent, policyMeta) {
        const { entityTypeMap } = this.config;
        const { mutedCacheOperations } = this;
        const typePolicyForEvent = this.getTypePolicyForEvent(typeName, policyEvent);
        if (!typePolicyForEvent) {
            return;
        }
        Object.keys(typePolicyForEvent).forEach((typeName) => {
            var _a;
            const typeMapEntities = (_a = entityTypeMap.readEntitiesByType(typeName)) !== null && _a !== void 0 ? _a : {};
            const policyAction = typePolicyForEvent[typeName];
            Object.values(typeMapEntities).forEach((typeMapEntity) => {
                const { dataId, fieldName, storeFieldNames } = typeMapEntity;
                if (storeFieldNames) {
                    Object.keys(storeFieldNames.entries).forEach((storeFieldName) => {
                        policyAction(mutedCacheOperations, Object.assign({ id: dataId, fieldName,
                            storeFieldName, variables: storeFieldNames.entries[storeFieldName].variables, ref: client_1.makeReference(dataId) }, policyMeta));
                    });
                }
                else {
                    policyAction(mutedCacheOperations, Object.assign({ id: dataId, ref: client_1.makeReference(dataId) }, policyMeta));
                }
            });
        });
    }
    getRenewalPolicyForType(typename) {
        var _a, _b, _c, _d;
        const { policies } = this.config;
        return ((_d = (_c = (_b = (_a = policies.types) === null || _a === void 0 ? void 0 : _a[typename]) === null || _b === void 0 ? void 0 : _b.renewalPolicy) !== null && _c !== void 0 ? _c : policies.renewalPolicy) !== null && _d !== void 0 ? _d : types_2.RenewalPolicy.WriteOnly);
    }
    runWritePolicy(typeName, policyMeta) {
        return this.runPolicyEvent(typeName, types_1.InvalidationPolicyEvent.Write, policyMeta);
    }
    runEvictPolicy(typeName, policyMeta) {
        return this.runPolicyEvent(typeName, types_1.InvalidationPolicyEvent.Evict, policyMeta);
    }
    runReadPolicy(typename, dataId, fieldName, storeFieldName) {
        var _a;
        const { cacheOperations, entityTypeMap, policies } = this.config;
        const entityId = helpers_1.makeEntityId(dataId, fieldName);
        const typeMapEntity = entityTypeMap.readEntityById(entityId);
        if (!typeMapEntity) {
            return true;
        }
        const entityCacheTime = storeFieldName && typeMapEntity.storeFieldNames
            ? typeMapEntity.storeFieldNames.entries[storeFieldName].cacheTime
            : typeMapEntity.cacheTime;
        const timeToLive = ((_a = this.getPolicy(typename)) === null || _a === void 0 ? void 0 : _a.timeToLive) || policies.timeToLive;
        if (lodash_1.default.isNumber(entityCacheTime) &&
            timeToLive &&
            Date.now() > entityCacheTime + timeToLive) {
            return cacheOperations.evict({
                id: dataId,
                fieldName: storeFieldName,
                broadcast: false,
            });
        }
        return false;
    }
    isPolicyActive(policyEvent) {
        return this.policyActivation[policyEvent];
    }
}
exports.default = InvalidationPolicyManager;
//# sourceMappingURL=InvalidationPolicyManager.js.map